%module Tess
%{
#include "tess.hh"
%}

%include typemaps.i
%apply int *REFERENCE { int& width, int& height };

typedef unsigned char ccomp;
struct Pixel {
    ccomp  red, green, blue, alpha;
};

void tess_setp (Pixel& pixel, ccomp r, ccomp g, ccomp b, ccomp a);

class TessSurface
{
public:

    enum TessDrawStyle { Blend = 1, Replace       };
    enum TessAlignment { Left  = 1, Right, Center };

    TessSurface (int width, int height);
    %name(TessSurface_reader) TessSurface (TessReader * reader);

    ~TessSurface ();

    void get_size (int& width, int& height);

    void set_alpha_pixel (int x, int y, ccomp alpha);
    void set_pixel (int x, int y, Pixel& pixel);
    void get_pixel (int x, int y, Pixel& pixel);
    void get_pma_pixel (int x, int y, Pixel& pixel);

    void set_type (TessType * type);
    void set_type_alignment (TessAlignment alignment);
    void set_type_inclination (int angle);
    void set_draw_width (int width);
    void set_draw_color (ccomp red, ccomp green, ccomp blue);
    void set_draw_style (TessDrawStyle style);

    void draw_string (int x, int y, char * string);
    void draw_uc_string (int x, int y, short * string);

    void draw_line (int x1, int y1, int x2, int y2);
    void draw_bezier (int x1, int y1, int x2, int y2, int x3, int y3,
		      int x4, int y4);
    void draw_rect (int x, int y, int width, int height);
    void draw_filled_rect (int x, int y, int width, int height);

    void copy_rect (int x, int y, int width, int height,
		    int dest_x, int dest_y);
    %name(copy_rect_surface)
    void copy_rect (TessSurface * surface, int x, int y, int width, int height,
	       int dest_x, int dest_y);

    void fade (int percent, int x, int y, int width, int height);
    void blur (int radius,  int x, int y, int width, int height);
    void flatten (ccomp red, ccomp green, ccomp blue);

    int write (TessWriter * writer);
};


class TessStack
{
public:

    TessStack ();
    ~TessStack ();

    void add_surface (TessSurface * surface, int x, int y);
    TessSurface * make_composite ();
};


class TessType
{
public:

    TessType (char * ttpath, int pointsize, int dpi_x, int dpi_y);
    ~TessType ();

    bool is_valid () { return (valid); }

    void set_rotation (int angle);
    void set_spacing (int spacing);

    int get_ascent ();
    int get_descent ();
    int get_spacing ();
    int get_string_width (char * string);
    int get_uc_string_width (short * string);

    void render_glyph (TessSurface * surface, int x, int y, int unicode);
    int get_glyph_width (int unicode);
    int get_pair_adjust (int unicode_left, int unicode_right);
};


class TessReader
{
public:

    virtual int get_size (int& width, int& height) = 0;
    virtual int populate (TessSurface * surface)   = 0;

    virtual ~TessReader () { }
};


class TessWriter
{
public:

    virtual void set_size (int width, int height) = 0;
    virtual int commit (TessSurface * surface)    = 0;

    virtual ~TessWriter () { }
};


class TessPPMReader : public TessReader
{
public:

    TessPPMReader (char * filepath, ccomp alpha = 255);
    virtual ~TessPPMReader ();
    
    virtual int get_size (int& width, int& height);
    virtual int populate (TessSurface * surface);
};


class TessPPMWriter : public TessWriter
{
public:

    TessPPMWriter (char * filepath);
    virtual ~TessPPMWriter ();

    virtual void set_size (int width, int height);
    virtual int commit (TessSurface * surface);
};


class TessPAMReader : public TessReader
{
public:

    TessPAMReader (char * filepath);
    virtual ~TessPAMReader ();
    
    virtual int get_size (int& width, int& height);
    virtual int populate (TessSurface * surface);
};


class TessPAMWriter : public TessWriter
{
public:

    TessPAMWriter (char * filepath);
    virtual ~TessPAMWriter ();

    virtual void set_size (int width, int height);
    virtual int commit (TessSurface * surface);
};
