//
// pam.cc
//
//  version 0.40  january 17, 1999
//  jeremy wohl (jeremyw@evantide.com)
//  no warranty, no responsibility, etc.
//

//
// Greg Roelofs (newt@pobox.com) has described an additional file format for
// the netpbm (formerly pbmplus) set of graphics file utilities,
// in http://www.wco.com/~png/greg/greg_rgba.html.
//
// I _do not_ implement his ppm+pgm approach, instead settling for
// incompatibility with existing netpbm code to make my own intermediate
// programs easier to write.  I add a 'P7' type, binary 8-bit rgba.
//

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include "tess.hh"
#include "config.hh"


TessPAMReader::TessPAMReader (char * filepath)
{
    file = fopen (filepath, "r");
}


TessPAMReader::~TessPAMReader ()
{
    if (file)  fclose (file);
}


int TessPAMReader::get_size (int& width, int& height)
{
    char  buf [10];


    fread (buf, 2, 1, file);
    if (buf [0] != 'P' && buf [1] != '7')
	return 1;

    this->width   = width  = read_int ();
    this->height  = height = read_int ();
    read_int ();

    return 0;
}


int TessPAMReader::read_int ()
{
    int i = 0, c;


    while (1) {
	if ((c = fgetc (file)) == EOF)  break;

	if (c == '#') {
	    while ((c = fgetc (file)) != EOF && c != '\r' && c != '\n')
		;
	}

	if (isdigit (c))  break;
    }

    do {
	i = i * 10 + c - '0';
    } while (isdigit ((c = fgetc (file))));

    return i;
}


int TessPAMReader::populate (TessSurface * surface)
{
    Pixel  pixel;
    int    row, col;


    for (row = 0; row < height; row++) {
	for (col = 0; col < width; col++) {
	    pixel.red   = fgetc (file);
	    pixel.green = fgetc (file);
	    pixel.blue  = fgetc (file);
	    pixel.alpha = fgetc (file);

	    surface->set_pixel (col, row, pixel);
	}
    }

    return 0;
}


TessPAMWriter::TessPAMWriter (char * filepath)
{
    this->filepath = new char [strlen (filepath) + 1];
    strcpy (this->filepath, filepath);
}


TessPAMWriter::~TessPAMWriter ()
{
    delete filepath;
}


void TessPAMWriter::set_size (int width, int height)
{
    this->width   = width;
    this->height  = height;
}


int TessPAMWriter::commit (TessSurface * surface)
{
    FILE  * file;
    Pixel   pixel;
    int     row, col;


    file = fopen (filepath, "w+");

    fprintf (file, "P7\n");
    fprintf (file, "# '%s' generated by tess.\n", filepath);
    fprintf (file, "%i %i\n", width, height);
    fprintf (file, "255\n");

    for (row = 0; row < height; row++) {
	for (col = 0; col < width; col++) {
	    surface->get_pixel (col, row, pixel);
	    fputc (pixel.red,   file);
	    fputc (pixel.green, file);
	    fputc (pixel.blue,  file);
	    fputc (pixel.alpha,  file);
	}
    }

    fclose (file);

    return (0);
}
