//
// ppm.cc
//
//  version 0.40  january 17, 1999
//  jeremy wohl (jeremyw@evantide.com)
//  no warranty, no responsibility, etc.
//

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include "tess.hh"
#include "config.hh"


TessPPMReader::TessPPMReader (char * filepath, ccomp alpha)
{
    file = fopen (filepath, "r");

    this->alpha = alpha;
}


TessPPMReader::~TessPPMReader ()
{
    if (file)  fclose (file);
}


int TessPPMReader::get_size (int& width, int& height)
{
    char  buf [10];


    fread (buf, 2, 1, file);
    if (buf [0] != 'P' && buf [1] != '6')
	return 1;

    this->width   = width  = read_int ();
    this->height  = height = read_int ();
    read_int ();

    return 0;
}


int TessPPMReader::read_int ()
{
    int i = 0, c;


    while (1) {
	if ((c = fgetc (file)) == EOF)  break;

	if (c == '#') {
	    while ((c = fgetc (file)) != EOF && c != '\r' && c != '\n')
		;
	}

	if (isdigit (c))  break;
    }

    do {
	i = i * 10 + c - '0';
    } while (isdigit ((c = fgetc (file))));

    return i;
}


int TessPPMReader::populate (TessSurface * surface)
{
    Pixel  pixel;
    int    row, col;


    pixel.alpha = alpha;

    for (row = 0; row < height; row++) {
	for (col = 0; col < width; col++) {
	    pixel.red   = fgetc (file);
	    pixel.green = fgetc (file);
	    pixel.blue  = fgetc (file);

	    surface->set_pixel (col, row, pixel);
	}
    }

    return 0;
}


TessPPMWriter::TessPPMWriter (char * filepath)
{
    this->filepath = new char [strlen (filepath) + 1];
    strcpy (this->filepath, filepath);
}


TessPPMWriter::~TessPPMWriter ()
{
    delete filepath;
}


void TessPPMWriter::set_size (int width, int height)
{
    this->width   = width;
    this->height  = height;
}


int TessPPMWriter::commit (TessSurface * surface)
{
    FILE  * file;
    Pixel   pixel;
    int     row, col;


    file = fopen (filepath, "w+");

    fprintf (file, "P6\n");
    fprintf (file, "# '%s' generated by tess.\n", filepath);
    fprintf (file, "%i %i\n", width, height);
    fprintf (file, "255\n");

    for (row = 0; row < height; row++) {
	for (col = 0; col < width; col++) {
	    surface->get_pixel (col, row, pixel);
	    fputc (pixel.red,   file);
	    fputc (pixel.green, file);
	    fputc (pixel.blue,  file);
	}
    }

    fclose (file);

    return (0);
}
