//
// stack.cc
//
//  version 0.40  january 17, 1999
//  jeremy wohl (jeremyw@evantide.com)
//  no warranty, no responsibility, etc.
//

#include "tess.hh"


TessStack::TessStack ()
{
    maxcol = 0;
    maxrow = 0;
    head   = 0;
}


TessStack::~TessStack ()
{
    node * hold1 = head, * hold2;

    while (hold1 && hold1->next) {
	hold2 = hold1->next;
	delete hold1;
	hold1 = hold2;
    }

    if (hold1)  delete hold1;
}


inline static int max (int a, int b) { return (a > b ? a : b); }

void TessStack::add_surface (TessSurface * surface, int x, int y)
{
    node * layer = head, * tmp = new node;


    while (layer && layer->next)
	layer = layer->next;

    tmp->surface  = surface;
    tmp->originx  = x;
    tmp->originy  = y;
    tmp->next     = 0;

    surface->get_size (tmp->width, tmp->height);
    maxcol = max (maxcol, tmp->width  + x);
    maxrow = max (maxrow, tmp->height + y);

    if (head == 0)
	head = tmp;
    else
	layer->next = tmp;
}


inline bool TessStack::is_layer_at (int col, int row, node * layer)
{
    if (col >= layer->originx && col < layer->originx + layer->width &&
	row >= layer->originy && row < layer->originy + layer->height)
	return true;
    else
	return false;
}

    
TessSurface * TessStack::make_composite ()
{
    TessSurface  * surface;
    Pixel          under, pixel;
    node         * layer;
    int            row, col;


    surface = new TessSurface (maxcol, maxrow);

    for (row = 0; row < maxrow; row++) {
	for (col = 0; col < maxcol; col++) {
	    layer = head;

	    tess_setp (under, 0, 0, 0, 0);
    
	    while (layer) {
		if (is_layer_at (col, row, layer)) {
		    layer->surface->get_pma_pixel
			(col - layer->originx, row - layer->originy, pixel);

		    if (pixel.alpha == 255 || under.alpha == 0)
			under = pixel;
		    else
			under = tess_posit (under, pixel);
		}

		layer = layer->next;
	    }

	    surface->set_pixel (col, row, under);
	}
    }

    return surface;
}
